@@ -24,9 +24,12 @@
 #include <chmapp.h>
 #include <chmframe.h>
 #include <chmfshandler.h>
+#include <wx/fs_inet.h>
 #include <chmfile.h>
 #include <wx/image.h>
 #include <wx/config.h>
+#include <wx/filefn.h>
+
 
 #ifdef WITH_LIBXMLRPC
 
@@ -143,18 +146,27 @@ bool CHMApp::OnInit()
 	long sashPos = CONTENTS_MARGIN;
 	long fontSize = CHM_DEFAULT_FONT_SIZE;
 	wxString lastOpenedDir, normalFont, fixedFont;
+#ifdef __WXMSW__
+	bool extreg = false;
+#endif
 
+#ifndef __WXMAC__
 	_loc.Init();
 	_loc.AddCatalog(wxT("xchm"));
+#endif
 
 	wxInitAllImageHandlers();
 	wxFileSystem::AddHandler(new CHMFSHandler);
+	wxFileSystem::AddHandler(new wxInternetFSHandler);
 
 	wxConfig config(wxT("xchm"));
 	if(config.Read(wxT("/Position/xOrig"), &xorig)) {
 		config.Read(wxT("/Position/yOrig"), &yorig);
 		config.Read(wxT("/Position/width"), &width);
 		config.Read(wxT("/Position/height"), &height);
+#ifdef __WXMSW__
+		config.Read(wxT("/Extensions/registered"), &extreg);
+#endif
 		config.Read(wxT("/Paths/lastOpenedDir"), 
 			    &lastOpenedDir);
 		config.Read(wxT("/Fonts/normalFontFace"), 
@@ -164,11 +176,16 @@ bool CHMApp::OnInit()
 		config.Read(wxT("/Sash/leftMargin"), &sashPos);
 	}
 
+	wxString fullAppPath;
+
+	if(argv > 0)
+		fullAppPath = getAppPath(argv[0], wxGetCwd());
+
 	_frame = new CHMFrame(wxT("xCHM v. " VERSION),
 				       lastOpenedDir, wxPoint(xorig, yorig), 
 				       wxSize(width, height), normalFont,
 				       fixedFont, static_cast<int>(fontSize),
-				       static_cast<int>(sashPos));
+				       static_cast<int>(sashPos), fullAppPath);
 
 	_frame->SetSizeHints(200, 200);
 	_frame->Show(TRUE);
@@ -196,6 +213,33 @@ bool CHMApp::OnInit()
 	return TRUE;
 }
 
+
+wxString CHMApp::getAppPath(const wxString& argv0, const wxString& cwd)
+{
+	if(wxIsAbsolutePath(argv0))
+		return argv0;
+
+	wxString cwdtmp(cwd), apppath;
+
+	if(cwdtmp.Last() != wxFILE_SEP_PATH)
+		cwdtmp += wxFILE_SEP_PATH;
+
+	apppath = cwdtmp + apppath;
+
+	if(wxFileExists(apppath))
+		return apppath;
+
+	wxPathList pathList;
+	pathList.AddEnvList(wxT("PATH"));
+	apppath = pathList.FindAbsoluteValidPath(argv0);
+	
+	if(!apppath.IsEmpty())
+		return wxPathOnly(apppath);
+
+	return wxEmptyString;
+}
+
+
 #ifdef __WXMAC__
 void CHMApp::MacOpenFile(const wxString& filename)
 {
@@ -220,3 +264,17 @@ END_EVENT_TABLE()
 // Apparently this macro gets main() pumping.
 IMPLEMENT_APP(CHMApp)
 
+
+/*
+  Local Variables:
+  mode: c++
+  c-basic-offset: 8
+  tab-width: 8
+  c-indent-comments-syntactically-p: t
+  c-tab-always-indent: t
+  indent-tabs-mode: t
+  End:
+*/
+
+// vim:shiftwidth=8:autoindent:tabstop=8:noexpandtab:softtabstop=8
+
