@@ -22,7 +22,6 @@
 
 
 #include <chmfile.h>
-#include <contenttaghandler.h>
 #include <chmlistctrl.h>
 #include <wx/wx.h>
 #include <wx/defs.h>
@@ -33,6 +32,7 @@
 #include <assert.h>
 
 #include <bitfiddle.inl>
+#include <hhcparser.h>
 
 
 namespace {
@@ -235,6 +235,8 @@ void CHMFile::CloseCHM()
 bool CHMFile::GetTopicsTree(wxTreeCtrl *toBuild)
 {
 	chmUnitInfo ui;
+	char buffer[BUF_SIZE];
+	size_t ret = BUF_SIZE - 1, curr = 0;
 
 	if(!toBuild)
 		return false;
@@ -242,16 +244,19 @@ bool CHMFile::GetTopicsTree(wxTreeCtrl *toBuild)
 	if(_topicsFile.IsEmpty() || !ResolveObject(_topicsFile, &ui))
 		return false;
 
-	wxString src;
-	src.Alloc(ui.length);
-	GetFileAsString(src, &ui);
+	buffer[0] = '\0';
 
-	if(src.IsEmpty())
-		return false;
+	HHCParser p(_enc, toBuild, NULL);
 
-	ContentParser parser;
-	parser.AddTagHandler(new ContentTagHandler(_enc, toBuild));	
-	parser.Parse(src);
+	do {
+		ret = RetrieveObject(&ui, reinterpret_cast<unsigned char *>(
+					     buffer), curr, BUF_SIZE - 1);
+		buffer[ret] = '\0';
+
+		p.parse(buffer);
+		curr += ret;
+
+	} while(ret == BUF_SIZE - 1);
 
 	return true;
 }
@@ -260,25 +265,28 @@ bool CHMFile::GetTopicsTree(wxTreeCtrl *toBuild)
 bool CHMFile::GetIndex(CHMListCtrl* toBuild)
 {
 	chmUnitInfo ui;
+	char buffer[BUF_SIZE];
+	size_t ret = BUF_SIZE - 1, curr = 0;
 
 	if(!toBuild)
 		return false;
 
-	if(_indexFile.IsEmpty() || !ResolveObject(_indexFile, &ui))
+	if(_topicsFile.IsEmpty() || !ResolveObject(_indexFile, &ui))
 		return false;
 
-	wxString src;
-	src.Alloc(ui.length);
-	GetFileAsString(src, &ui);
+	buffer[0] = '\0';
 
-	if(src.IsEmpty())
-		return false;
+	HHCParser p(_enc, NULL, toBuild);
 
-	ContentParser parser;
-	parser.AddTagHandler(new ContentTagHandler(_enc, NULL, toBuild));
+	do {
+		ret = RetrieveObject(&ui, reinterpret_cast<unsigned char *>(
+					     buffer), curr, BUF_SIZE - 1);
+		buffer[ret] = '\0';
 
-	parser.Parse(src);
-	toBuild->UpdateUI();
+		p.parse(buffer);
+		curr += ret;
+
+	} while(ret == BUF_SIZE - 1);
 
 	return true;
 }
@@ -712,27 +720,6 @@ bool CHMFile::ProcessWLC(u_int64_t wlc_count, u_int64_t wlc_size,
 }
 
 
-inline
-void CHMFile::GetFileAsString(wxString& str, chmUnitInfo *ui)
-{
-#define BUF_SIZE2 1025
-	char buffer[BUF_SIZE2];
-	size_t ret = BUF_SIZE2 - 1, curr = 0;
-
-	buffer[0] = '\0';
-
-	do {
-		ret = RetrieveObject(ui, reinterpret_cast<unsigned char *>(
-					     buffer), curr, BUF_SIZE2 - 1);
-		buffer[ret] = '\0';
-
-		str.Append(CURRENT_CHAR_STRING(buffer));
-		curr += ret;
-
-	} while(ret == BUF_SIZE2 - 1);
-}
-
-
 inline
 bool CHMFile::InfoFromWindows()
 {
@@ -1081,3 +1068,16 @@ wxFontEncoding CHMFile::GetFontEncFromLCID(u_int32_t lcid)
 }
 
 
+/*
+  Local Variables:
+  mode: c++
+  c-basic-offset: 8
+  tab-width: 8
+  c-indent-comments-syntactically-p: t
+  c-tab-always-indent: t
+  indent-tabs-mode: t
+  End:
+*/
+
+// vim:shiftwidth=8:autoindent:tabstop=8:noexpandtab:softtabstop=8
+
