@@ -46,25 +46,6 @@ WX_DECLARE_HASH_MAP( int, wxString, wxIntegerHash, wxIntegerEqual, CHMIDMap );
 #define MAX_SEARCH_RESULTS 512
 
 
-#if wxUSE_UNICODE
-
-#	define CURRENT_CHAR_STRING(x) \
-	wxString(reinterpret_cast<const char *>(x), wxConvISO8859_1)
-
-#	define CURRENT_CHAR_STRING_CV(x, cv) \
-	wxString(reinterpret_cast<const char *>(x), cv)
-
-#else
-
-#	define CURRENT_CHAR_STRING(x) \
-	wxString(reinterpret_cast<const char *>(x))
-
-#	define CURRENT_CHAR_STRING_CV(x, cv) \
-	wxString(reinterpret_cast<const char *>(x))
-
-
-#endif
-
 
 //! Mostly a C++ wrapper around the CHMLIB facilities. Concrete class.
 class CHMFile {
@@ -274,9 +255,6 @@ private:
 			chmUnitInfo* topics, chmUnitInfo *urlstr,
 			CHMSearchResults *results);
 
-	//! Puts in the str parameter the contents of the file referred by ui.
-	void GetFileAsString(wxString& str, chmUnitInfo *ui);
-
 	//! Looks up as much information as possible from #WINDOWS/#STRINGS.
 	bool InfoFromWindows();
 
@@ -304,3 +282,18 @@ private:
 
 
 #endif // __CHMFILE_H_
+
+
+/*
+  Local Variables:
+  mode: c++
+  c-basic-offset: 8
+  tab-width: 8
+  c-indent-comments-syntactically-p: t
+  c-tab-always-indent: t
+  indent-tabs-mode: t
+  End:
+*/
+
+// vim:shiftwidth=8:autoindent:tabstop=8:noexpandtab:softtabstop=8
+
