@@ -61,7 +61,6 @@ CHMFindDialog::CHMFindDialog(wxWindow *parent, CHMHtmlWindow *toSearch)
 	topsizer->Fit(this);
 	Centre(wxBOTH);
 
-	find->SetDefault();
 	SetFocusToTextBox();
 }
 
@@ -126,3 +125,17 @@ BEGIN_EVENT_TABLE(CHMFindDialog, wxDialog)
 END_EVENT_TABLE()
 
 
+/*
+  Local Variables:
+  mode: c++
+  c-basic-offset: 8
+  tab-width: 8
+  c-indent-comments-syntactically-p: t
+  c-tab-always-indent: t
+  indent-tabs-mode: t
+  End:
+*/
+
+// vim:shiftwidth=8:autoindent:tabstop=8:noexpandtab:softtabstop=8
+
+
