@@ -25,15 +25,17 @@
 
 #include <chmframe.h>
 #include <chminputstream.h>
-#include <contenttaghandler.h>
 #include <chmhtmlwindow.h>
 #include <chmfontdialog.h>
 #include <chmsearchpanel.h>
 #include <chmindexpanel.h>
 #include <chmlistctrl.h>
+#include <hhcparser.h>
 #include <wx/fontenum.h>
 #include <wx/statbox.h>
 #include <wx/accel.h>
+#include <wx/filesys.h>
+#include <wx/mimetype.h>
 
 
 #define OPEN_HELP _("Open a CHM book.")
@@ -44,6 +46,10 @@
 #define FORWARD_HELP _("Go forward in history. Per book.")
 #define BACK_HELP _("Back to the last visited page. Per book.")
 #define ABOUT_HELP _("About the program.")
+#define COPY_HELP _("Copy selection.")
+#define FIND_HELP _("Find word in page.")
+#define REGISTER_EXTENSION_HELP _("Associate the .chm file " \
+				  "extension with xCHM.")
 
 
 namespace {
@@ -57,10 +63,10 @@ const wxChar *greeting = wxT(
 	"wxwidgets.org\">wxWidgets</a>.<br><br>If you'd like to know more"
 	" about CHM, you could check out <a href=\"http://www.speakeasy."
 	"org/~russotto/chm/\">Matthew Russoto's CHM page</a> or <a"
-	" href=\"http://bonedaddy.net/pabs3/hhm/\">Pabs' CHM Specification"
-	" page</a>.<br>Pabs has contributed time and knowledge to the"
-	" development of <B>xCHM</B>, and features such as the fast index"
-	" search would not have been implemented without his help."
+	" href=\"http://www.nongnu.org/chmspec/latest/\">Pabs' CHM"
+	" Specification page</a>.<br>Pabs has contributed time and knowledge"
+	" to the development of <B>xCHM</B>, and features such as the fast"
+	" index search would not have been implemented without his help."
 	" Portions of the fast index search are modified versions of"
 	" Pabs' <TT>GPL</TT>d <TT>chmdeco</TT> code.<br><br>If"
 	" you'd like to use the code in your own stuff please figure"
@@ -74,7 +80,12 @@ const wxChar *greeting = wxT(
 	" for \'ans 4\' (quotes not included) will find pages that"
 	" contain the sentence \'The answer is 42.\'.</li><li>Right"
 	" clicking on the displayed page brings out a popup menu with"
-	" common options.</li></ul><br><br>Enjoy.</body></html>");
+	" common options.</li></ul><br><br>Ctrl(cmd)-C is copy,"
+	" Ctrl(cmd)-F is find in page.<br><br>Enjoy.</body></html>");
+
+
+const wxChar *error_page = wxT(
+	"<html><body>Error loading CHM file!</body></html>");
 
 
 const wxChar *about_txt = wxT(
@@ -93,25 +104,31 @@ const wxChar *about_txt = wxT(
 } // namespace
 
 
-
 CHMFrame::CHMFrame(const wxString& title, const wxString& booksDir, 
 		   const wxPoint& pos, const wxSize& size,
 		   const wxString& normalFont, const wxString& fixedFont,
-		   const int fontSize, const int sashPosition)
+		   const int fontSize, const int sashPosition,
+		   const wxString& fullAppPath)
 	: wxFrame(NULL, -1, title, pos, size), _html(NULL),
 	  _tcl(NULL), _sw(NULL), _menuFile(NULL), _tb(NULL), _ep(NULL),
 	  _nb(NULL), _cb(NULL), _csp(NULL), _cip(NULL), _openPath(booksDir), 
 	  _normalFonts(NULL), _fixedFonts(NULL), _normalFont(normalFont), 
 	  _fixedFont(fixedFont), _fontSize(fontSize), _bookmarkSel(true),
-	  _bookmarksDeleted(false), _sashPos(sashPosition)
+	  _bookmarksDeleted(false), _sashPos(sashPosition),
+	  _fullAppPath(fullAppPath)
 {
 #	if wxUSE_ACCEL
-	wxAcceleratorEntry entries[1];
+	wxAcceleratorEntry entries[2];
 	entries[0].Set(wxACCEL_CTRL, (int) 'F', ID_FindInPage);
-	wxAcceleratorTable accel(1, entries);
+	entries[1].Set(wxACCEL_CTRL, (int) 'C', ID_CopySelection);
+	wxAcceleratorTable accel(2, entries);
 	SetAcceleratorTable(accel);
 #	endif
 
+#ifdef __WXMAC__	
+	wxApp::s_macAboutMenuItemId = ID_About;
+#endif
+
 	int sizes[7];
 	for(int i = -3; i <= 3; ++i)
 		sizes[i+3] = _fontSize + i * 2;
@@ -264,7 +281,14 @@ void CHMFrame::OnHistoryForward(wxCommandEvent& WXUNUSED(event))
 
 void CHMFrame::OnHistoryBack(wxCommandEvent& WXUNUSED(event))
 {
-	_html->HistoryBack();
+	if(_html->HistoryCanBack())
+		_html->HistoryBack();
+	else {
+		if(CHMInputStream::GetCache())
+			return;
+		
+		_html->SetPage(greeting);
+	}
 }
 
 
@@ -279,7 +303,7 @@ void CHMFrame::OnShowContents(wxCommandEvent& WXUNUSED(event))
 		_nb->Show(FALSE);
 	} else {		
 			
-		if(_tcl->GetCount() >= 1) {
+		//if(_tcl->GetCount() >= 1) {
 			
 			_tb->ToggleTool(ID_Contents, TRUE);
 			_menuFile->Check(ID_Contents, TRUE);
@@ -287,7 +311,7 @@ void CHMFrame::OnShowContents(wxCommandEvent& WXUNUSED(event))
 			_nb->Show(TRUE);
 			_sw->SplitVertically(_nb, _html, _sashPos);
 
-		} else {
+		/*} else {
 			_tb->ToggleTool(ID_Contents, FALSE);
 			_menuFile->Check(ID_Contents, FALSE);
 			
@@ -295,9 +319,37 @@ void CHMFrame::OnShowContents(wxCommandEvent& WXUNUSED(event))
 				       " contents tree."), 
 				       _("No contents.."), 
 				       wxOK | wxICON_WARNING, this );
+	       }*/
+	}
+}
+
+
+#if defined(__WXMSW__) || defined(__WXMAC__)
+void CHMFrame::OnRegisterExtension(wxCommandEvent& WXUNUSED(event))
+{
+	int answer = wxMessageBox(_("This is experimental code, and "
+				    "doing this will overwrite any "
+				    "previously registered CHM viewer "
+				    "associations.\n\nAre you sure "
+				    "you know what you're doing?"),
+				  _("Confirm"),
+				  wxOK | wxCANCEL, this);
+
+	if(answer == wxOK) {
+		wxFileTypeInfo fti(wxT("application/x-chm"), _fullAppPath,
+				   wxEmptyString, wxT("Compiled HTML help"),
+				   wxT("chm"), NULL);
+
+		wxFileType *ft = wxTheMimeTypesManager->Associate(fti);
+
+		if(ft) {
+			ft->SetDefaultIcon(_fullAppPath);
+			wxMessageBox(_("Registration successful!"), 
+				     _("Done"), wxOK, this);
 		}
 	}
 }
+#endif// __WXMSW__
 
 
 void CHMFrame::OnPrint(wxCommandEvent& WXUNUSED(event))
@@ -325,6 +377,12 @@ void CHMFrame::OnFind(wxCommandEvent& event)
 }
 
 
+void CHMFrame::OnCopySelection(wxCommandEvent& event)
+{
+	_html->OnCopy(event);
+}
+
+
 void CHMFrame::OnAddBookmark(wxCommandEvent& WXUNUSED(event))
 {
 	wxTreeItemId id = _tcl->GetSelection();
@@ -369,16 +427,16 @@ void CHMFrame::OnRemoveBookmark(wxCommandEvent& WXUNUSED(event))
 }
 
 
-void CHMFrame::OnBookmarkSel(wxCommandEvent& WXUNUSED(event))
+void CHMFrame::OnBookmarkSel(wxCommandEvent& event)
 {
 	if(!_bookmarkSel)
 		return;
 
 	wxString *url = reinterpret_cast<wxString *>(
 #ifdef __WXGTK__
-		_cb->GetClientData(_cb->GetSelection()));
+		_cb->GetClientData(event.GetSelection()));
 #else
-		_cb->wxItemContainer::GetClientData(_cb->GetSelection()));
+		_cb->wxItemContainer::GetClientData(event.GetSelection()));
 #endif
 
 	if(!url || url->IsEmpty())
@@ -431,12 +489,27 @@ bool CHMFrame::LoadCHM(const wxString& archive)
 	bool rtn;
 	SaveBookmarks();
 	_nb->SetSelection(0);
-	rtn = _html->LoadPage(wxString(wxT("file:")) + archive +
-		      wxT("#xchm:/"));
 
+	wxFileSystem wfs;
+	wxFSFile *f = wfs.OpenFile(wxString(wxT("file:")) + archive +
+		      wxT("#xchm:/"));
+	delete f;
+	
+        CHMFile *chmf = CHMInputStream::GetCache();
+
+	if(!chmf)
+		return false;
+	
+	rtn = _html->LoadPage(wxString(wxT("file:"))
+					+ chmf->ArchiveName()
+			                + wxT("#xchm:") 
+					+ chmf->HomePage());
 	UpdateCHMInfo();
 	LoadBookmarks();
 
+	if(!rtn)
+		_html->SetPage(error_page);
+
 	return rtn;
 }
 
@@ -482,12 +555,12 @@ void CHMFrame::UpdateCHMInfo()
 	_cip->Reset();
 
 	wxString title = chmf->Title();
-	
+
 	if(_tcl->GetCount())
 		_tcl->DeleteAllItems();
 	chmf->GetTopicsTree(_tcl);
 	chmf->GetIndex(_cip->GetResultsList());
- 
+	
 	if(!title.IsEmpty()) {
 		wxString titleBarText = 
 			wxString(wxT("xCHM v. " VERSION ": ")) + title;
@@ -588,6 +661,13 @@ wxMenuBar* CHMFrame::CreateMenu()
 				   CONTENTS_HELP);
 	_menuFile->AppendSeparator();
 
+#if defined(__WXMSW__) || defined(__WXMAC__)
+	_menuFile->Append(ID_RegisterExtension, 
+			  _("&Make xCHM the default CHM viewer"),
+			  REGISTER_EXTENSION_HELP);
+	_menuFile->AppendSeparator();
+#endif
+
 	wxMenu *recent = new wxMenu;
 	_menuFile->Append(ID_Recent, _("&Recent files"), recent);
 	_fh.UseMenu(recent);
@@ -611,17 +691,23 @@ wxMenuBar* CHMFrame::CreateMenu()
 	menuHistory->Append(ID_Back, _("&Back\tCtrl-B"), BACK_HELP);
 	
 	wxMenu *menuHelp = new wxMenu;
-	menuHelp->Append(ID_About, _("&About ..\tF1"), ABOUT_HELP);
+	menuHelp->Append(ID_About, _("&About..\tF1"), ABOUT_HELP);
+
+	wxMenu *menuEdit = new wxMenu;
+	menuEdit->Append(ID_CopySelection, _("&Copy\tCtrl-C"), COPY_HELP);
+	menuEdit->Append(ID_FindInPage, _("&Find..\tCtrl-F"), FIND_HELP);
 
 	wxMenuBar *menuBar = new wxMenuBar;
 	menuBar->Append(_menuFile, _("&File"));
+	menuBar->Append(menuEdit, _("&Edit"));
 	menuBar->Append(menuHistory, _("Hi&story"));
 	menuBar->Append(menuHelp, _("&Help"));
 
+
 	return menuBar;
 }
 
-
+//#ifndef __WXMSW__
 namespace {
 
 #include <hbook_open.xpm>
@@ -629,6 +715,7 @@ namespace {
 #include <hpage.xpm>
 
 } // namespace
+//#endif
 
 
 wxPanel* CHMFrame::CreateContentsPanel()
@@ -642,6 +729,7 @@ wxPanel* CHMFrame::CreateContentsPanel()
 	temp->SetAutoLayout(TRUE);
         temp->SetSizer(sizer);
 
+#ifndef __WXMSW__
 	wxImageList *tclBL = new wxImageList(16, 16);
 	tclBL->Add(wxIcon(hbook_closed_xpm));
 	tclBL->Add(wxIcon(hbook_closed_xpm));
@@ -649,12 +737,27 @@ wxPanel* CHMFrame::CreateContentsPanel()
 	tclBL->Add(wxIcon(hbook_open_xpm));
 	wxImageList *tclIL = new wxImageList(16, 16);
 	tclIL->Add(wxIcon(hpage_xpm));
+#else
+	wxImageList *il = new wxImageList(16, 16);
+	il->Add(wxIcon(hbook_closed_xpm));
+	il->Add(wxIcon(hbook_open_xpm));
+	il->Add(wxIcon(hpage_xpm));
+#endif
+
 	_tcl = new wxTreeCtrl(temp, ID_TreeCtrl, wxDefaultPosition, 
-			      wxDefaultSize, wxTR_HAS_BUTTONS
-			      | wxSUNKEN_BORDER | wxTR_HIDE_ROOT
+			      wxDefaultSize, 
+#ifndef __WXMSW__
+			      wxTR_HAS_BUTTONS |
+#endif
+			      wxSUNKEN_BORDER | wxTR_HIDE_ROOT
 			      | wxTR_LINES_AT_ROOT);
+
+#ifndef __WXMSW__
 	_tcl->AssignButtonsImageList(tclBL);
 	_tcl->AssignImageList(tclIL);
+#else
+	_tcl->AssignImageList(il);	
+#endif
 
 	_cb = new wxComboBox(temp, ID_Bookmarks, wxT(""), wxDefaultPosition,
 			     wxDefaultSize, 0, NULL, wxCB_DROPDOWN 
@@ -796,6 +899,8 @@ namespace {
 #include <helpicon.xpm>
 #include <htmsidep.xpm>
 #include <htmoptns.xpm>
+#include <copy.xpm>
+#include <find.xpm>
 
 } // namespace
 
@@ -811,6 +916,13 @@ bool CHMFrame::InitToolBar(wxToolBar *toolbar)
 	toolbar->AddCheckTool(ID_Contents, _("Contents"),
 			      wxBitmap(htmsidep_xpm),
 			      wxBitmap(htmsidep_xpm), CONTENTS_HELP);
+	
+	toolbar->AddSeparator();
+	toolbar->AddTool(ID_CopySelection, _("Copy"), 
+			wxBitmap(copy_xpm), COPY_HELP);
+	toolbar->AddTool(ID_FindInPage, _("Find"), 
+			wxBitmap(find_xpm), FIND_HELP);
+	
 	toolbar->AddSeparator();
 
 	toolbar->AddTool(ID_Back, _("Back"), wxBitmap(back_xpm), BACK_HELP);
@@ -838,9 +950,13 @@ BEGIN_EVENT_TABLE(CHMFrame, wxFrame)
 	EVT_MENU(ID_Forward, CHMFrame::OnHistoryForward)
 	EVT_MENU(ID_Back, CHMFrame::OnHistoryBack)
 	EVT_MENU(ID_Contents, CHMFrame::OnShowContents)
+#if defined(__WXMSW__) || defined(__WXMAC__)
+	EVT_MENU(ID_RegisterExtension, CHMFrame::OnRegisterExtension)
+#endif
 	EVT_MENU(ID_Print, CHMFrame::OnPrint)
 	EVT_MENU_RANGE(wxID_FILE1, wxID_FILE9, CHMFrame::OnHistFile)
 	EVT_MENU(ID_FindInPage, CHMFrame::OnFind)
+	EVT_MENU(ID_CopySelection, CHMFrame::OnCopySelection)
 	EVT_BUTTON(ID_Add, CHMFrame::OnAddBookmark)
 	EVT_BUTTON(ID_Remove, CHMFrame::OnRemoveBookmark)
 	EVT_TREE_SEL_CHANGED(ID_TreeCtrl, CHMFrame::OnSelectionChanged)
@@ -849,4 +965,16 @@ BEGIN_EVENT_TABLE(CHMFrame, wxFrame)
 END_EVENT_TABLE()
 
 
+/*
+  Local Variables:
+  mode: c++
+  c-basic-offset: 8
+  tab-width: 8
+  c-indent-comments-syntactically-p: t
+  c-tab-always-indent: t
+  indent-tabs-mode: t
+  End:
+*/
+
+// vim:shiftwidth=8:autoindent:tabstop=8:noexpandtab:softtabstop=8
 
