@@ -22,20 +22,22 @@
 
 
 #include <chmhtmlwindow.h>
-#include <contenttaghandler.h>
+#include <hhcparser.h>
 #include <chminputstream.h>
 #include <chmframe.h>
 #include <wx/wx.h>
 #include <wx/log.h>
 #include <wx/clipbrd.h>
+#include <wx/filename.h>
+#include <wx/uri.h>
 
 
 
-CHMHtmlWindow::CHMHtmlWindow(wxWindow *parent, wxTreeCtrl *tc,
-			     CHMFrame *frame)
+
+CHMHtmlWindow::CHMHtmlWindow(wxWindow *parent, wxTreeCtrl *tc, CHMFrame *frame)
 	: wxHtmlWindow(parent, -1, wxDefaultPosition, wxSize(200,200)),
 	  _tcl(tc), _syncTree(true), _found(false), _menu(NULL), 
-	  _absolute(false), _frame(frame), _link(NULL), _fdlg(NULL)
+	  _absolute(false), _frame(frame), _fdlg(NULL)
 {
 	_menu = new wxMenu;
 	_menu->Append(ID_PopupForward, _("For&ward"));
@@ -58,15 +60,15 @@ CHMHtmlWindow::~CHMHtmlWindow()
 
 bool CHMHtmlWindow::LoadPage(const wxString& location)
 {
-	//wxLogNull log;
+	wxLogNull log;
 	wxString tmp = location;
 
 	// Path is already absolute.
 	if(_absolute) {
-		FixRelativePath(tmp, wxEmptyString);
+		FixPath(tmp, wxT("/"));
 		_absolute = false;
 	} else
-		FixRelativePath(tmp, GetPrefix(GetOpenedPage()));
+		FixPath(tmp, _prefix);
 
 	if(!tmp.Left(19).CmpNoCase(wxT("javascript:fullsize"))) 
 		tmp = tmp.AfterFirst(wxT('\'')).BeforeLast(wxT('\''));
@@ -130,57 +132,78 @@ void CHMHtmlWindow::Sync(wxTreeItemId root, const wxString& page)
 }
 
 
-inline 
 wxString CHMHtmlWindow::GetPrefix(const wxString& location) const
 {
 	return location.AfterLast(wxT(':')).AfterFirst(
-		wxT('/')).BeforeLast(wxT('/'));
+		           wxT('/')).BeforeLast(wxT('/'));
 }
 
 
-bool CHMHtmlWindow::FixRelativePath(wxString &location,
-				    const wxString& prefix) const
+bool CHMHtmlWindow::FixPath(wxString &location,
+			    const wxString& prefix) const
 {
-	if(!location.Left(5).CmpNoCase(wxT("file:")) || 
-	   !location.Left(5).CmpNoCase(wxT("http:")) ||
-	   !location.Left(6).CmpNoCase(wxT("https:")) ||
-	   !location.Left(4).CmpNoCase(wxT("ftp:")) ||
-	   !location.Left(7).CmpNoCase(wxT("mailto:")) ||
-	   !location.Left(7).CmpNoCase(wxT("ms-its:")) ||
-	   !location.Left(10).CmpNoCase(wxT("javascript")) ||
-	   location.StartsWith(wxT("#")))
-		return false;
-	  
+	wxLogNull wln;
+
+        if(!location.Left(5).CmpNoCase(wxT("http:")) ||
+           !location.Left(6).CmpNoCase(wxT("https:")) ||
+           !location.Left(4).CmpNoCase(wxT("ftp:")) || 
+           !location.Left(7).CmpNoCase(wxT("mailto:")) ||
+           !location.Left(7).CmpNoCase(wxT("ms-its:")) ||
+           !location.Left(10).CmpNoCase(wxT("javascript")) ||
+           location.StartsWith(wxT("#")))
+                return false;
+
 	CHMFile *chmf = CHMInputStream::GetCache();
 
 	if(!chmf)
 		return false;
 
-	bool result = location.StartsWith(wxT(".."));
-	wxString prf = prefix;
+	bool full = location.StartsWith(wxT("file:"));
+	bool repairWX = false;
+	size_t len = 0;
 
-	while(location.StartsWith(wxT(".."))) {
-		location = location.AfterFirst(wxT('/'));
-		prf = prf.BeforeLast(wxT('/'));
+	if(full) {
+		wxString basepath = GetParser()->GetFS()->GetPath();
+		len = basepath.Length();
+		
+		repairWX = location.StartsWith(basepath)
+			&& len < location.Length()
+			&& location[len] == wxT('/');
 	}
 
-	if(location.StartsWith(wxT("./")))
-		location = location.AfterFirst(wxT('/'));
-		
-	if(!prf.IsEmpty())
-		prf = wxString(wxT("/")) + prf;
+	if(!repairWX && full && !location.Contains(wxT("./")))
+		return false;
+	
+	wxString arch;
+	wxString file;
 
-	// Handle absolute paths too (that start with /)
-	if(!location.StartsWith(wxT("/")))
-		location = wxString(wxT("file:")) + chmf->ArchiveName() + 
-			wxT("#xchm:") + prf + wxT("/") + location;
+	if(full)
+		arch = location.BeforeFirst(wxT('#')).Mid(5);
 	else
-		location = wxString(wxT("file:")) + chmf->ArchiveName() + 
-			wxT("#xchm:") + location;
+		arch = chmf->ArchiveName();			
 
-	// Don't redirect for simple relative paths that wxHTML can
-	// handle by itself.
-	return result;
+	wxFileName awfn(arch);
+	awfn.Normalize();
+
+	if(full) {
+		if(!repairWX)
+			file = location.AfterFirst(wxT('#')).
+				AfterFirst(wxT(':'));
+		else 
+			file = location.Mid(len);
+	} else 
+		file = location;
+
+	if(!file.StartsWith(wxT("/"))) 
+		file = wxString(wxT("/")) + prefix + wxT("/") + file;
+
+	wxFileName fwfn(file, wxPATH_UNIX);
+	fwfn.Normalize(wxPATH_NORM_DOTS);
+
+	location = wxString(wxT("file:")) + awfn.GetFullPath() +
+		wxString(wxT("#xchm:")) + fwfn.GetFullPath(wxPATH_UNIX);
+	
+	return true;
 }
 
 
@@ -193,7 +216,7 @@ wxHtmlOpeningStatus CHMHtmlWindow::OnOpeningURL(wxHtmlURLType type,
 
 	wxString tmp = url;
 
-	if(FixRelativePath(tmp, _prefix)) {
+	if(FixPath(tmp, _prefix)) {
 		*redirect = tmp;
 		return wxHTML_REDIRECT;
 	}
@@ -205,7 +228,7 @@ wxHtmlOpeningStatus CHMHtmlWindow::OnOpeningURL(wxHtmlURLType type,
 void CHMHtmlWindow::OnLinkClicked(const wxHtmlLinkInfo& link)
 {
 	wxString url = link.GetHref();
-
+	
 	LoadPage(url);
 
 	if(!url.Left(7).CmpNoCase(wxT("MS-ITS:")))
@@ -350,7 +373,7 @@ void CHMHtmlWindow::OnCopyLink(wxCommandEvent& WXUNUSED(event))
 {
 	if(wxTheClipboard->Open()) {
 		wxTheClipboard->SetData(
-			new wxTextDataObject(_link->GetHref()));
+			new wxTextDataObject(_link));
 		wxTheClipboard->Close();
 	}
 }
@@ -365,70 +388,37 @@ void CHMHtmlWindow::OnRightClick(wxMouseEvent& event)
 	_menu->Enable(ID_PopupBack, HistoryCanBack());
 	_menu->Enable(ID_CopyLink, false);
 
-	wxHtmlCell *cell = 
-		m_Cell->FindCellByPos(event.m_x, event.m_y);
+        int x, y;
+        CalcUnscrolledPosition(event.m_x, event.m_y, &x, &y);
 
-	if(cell) {
-		_link = cell->GetLink();
+	wxHtmlCell *cell = GetInternalRepresentation()->
+		FindCellByPos(x, y);
 
-		if(_link) {
-			_menu->Enable(ID_CopyLink, true);
-		}
+	wxHtmlLinkInfo* linfo = NULL;
+
+	if(cell)
+		linfo = cell->GetLink();
+
+	if(linfo) {
+		_link = linfo->GetHref();
+		_menu->Enable(ID_CopyLink, true);
 	}
 
 	PopupMenu(_menu, event.GetPosition());
 }
 
 
-#ifdef __WXMAC__
-/* This is a hack, consisting of copying the relevent event handler from
-   the wxWidgets src/generic/scrlwing.cpp file to make up for the fact that for
-   some reason, compiling wxWidgets with scroll wheel support is exposing a 
-   crashing bug on OS X.
-   When this gets fixed in wxWidgets, this should prob be just removed.. */
-void CHMHtmlWindow::HandleOnMouseWheel(wxMouseEvent& event)
+void CHMHtmlWindow::OnChar(wxKeyEvent& event)
 {
-	int m_wheelRotation = 0;
-	m_wheelRotation += event.GetWheelRotation();
-	int lines = m_wheelRotation / event.GetWheelDelta();
-	m_wheelRotation -= lines * event.GetWheelDelta();
+	if(event.GetKeyCode() == WXK_SPACE) {
+		event.m_keyCode = WXK_PAGEDOWN;
 
-	if (lines != 0)
-	{
-		wxScrollWinEvent newEvent;
-
-		newEvent.SetPosition(0);
-		newEvent.SetOrientation(wxVERTICAL);
-		newEvent.m_eventObject = m_win;
-
-		if (event.IsPageScroll())
-		{
-			if (lines > 0)
-				newEvent.m_eventType = 
-					wxEVT_SCROLLWIN_PAGEUP;
-			else
-				newEvent.m_eventType = 
-					wxEVT_SCROLLWIN_PAGEDOWN;
-
-			m_win->GetEventHandler()->ProcessEvent(newEvent);
-		} else {
-			lines *= event.GetLinesPerAction();
-			if (lines > 0)
-				newEvent.m_eventType = 
-					wxEVT_SCROLLWIN_LINEUP;
-			else
-				newEvent.m_eventType = 
-					wxEVT_SCROLLWIN_LINEDOWN;
-
-			int times = abs(lines);
-			for (; times > 0; times--)
-				m_win->GetEventHandler()
-					->ProcessEvent(newEvent);
-		}
+	} else if(event.GetKeyCode() == WXK_BACK) {
+		event.m_keyCode = WXK_PAGEUP;
 	}
-}
-#endif
 
+	event.Skip();
+}
 
 
 BEGIN_EVENT_TABLE(CHMHtmlWindow, wxHtmlWindow)
@@ -437,10 +427,22 @@ BEGIN_EVENT_TABLE(CHMHtmlWindow, wxHtmlWindow)
 	EVT_MENU(ID_PopupForward, CHMHtmlWindow::OnForward)
 	EVT_MENU(ID_PopupBack, CHMHtmlWindow::OnBack)
 	EVT_MENU(ID_CopyLink, CHMHtmlWindow::OnCopyLink)
+	EVT_CHAR(CHMHtmlWindow::OnChar)
 	EVT_RIGHT_DOWN(CHMHtmlWindow::OnRightClick)
-#ifdef __WXMAC__
-	EVT_MOUSEWHEEL(CHMHtmlWindow::HandleOnMouseWheel)
-#endif
 END_EVENT_TABLE()
 
 
+/*
+  Local Variables:
+  mode: c++
+  c-basic-offset: 8
+  tab-width: 8
+  c-indent-comments-syntactically-p: t
+  c-tab-always-indent: t
+  indent-tabs-mode: t
+  End:
+*/
+
+// vim:shiftwidth=8:autoindent:tabstop=8:noexpandtab:softtabstop=8
+
+
