@@ -124,10 +124,10 @@ public:
 	//! Called when the user selects 'Find' from the popup menu.
 	void OnFind(wxCommandEvent& event);
 
-protected:	
 	//! Called when the user selects 'Copy' from the popup menu.
 	void OnCopy(wxCommandEvent& event);
-	
+
+protected:	
 	//! Called when the user selects 'Forward' from the popup menu.
 	void OnForward(wxCommandEvent& event);
 
@@ -137,13 +137,12 @@ protected:
 	//! Called when the user selects 'Copy link' from the popup menu.
 	void OnCopyLink(wxCommandEvent& event);
 
+	//! Called when the user presses a key
+	void OnChar(wxKeyEvent &event);
+
 protected:	
 	//! Called when the user right clicks the HTML window.
 	void OnRightClick(wxMouseEvent& event);
-#ifdef __WXMAC__
-	//! Handles scrollwheel support.
-	void HandleOnMouseWheel(wxMouseEvent& event);
-#endif 
 	//! Overridden to correct relative images paths.
 	wxHtmlOpeningStatus OnOpeningURL(wxHtmlURLType type,
 					 const wxString& url,
@@ -160,7 +159,7 @@ private:
 	wxString GetPrefix(const wxString& location) const;
 
 	//! Helper. Fixes ../dir/file locations.
-	bool FixRelativePath(wxString& location, const wxString& prefix) const;
+	bool FixPath(wxString& location, const wxString& prefix) const;
 	
 private:
 	wxTreeCtrl* _tcl;
@@ -170,7 +169,7 @@ private:
 	wxString _prefix;
 	bool _absolute;
 	CHMFrame *_frame;
-	wxHtmlLinkInfo *_link;
+	wxString _link;
 	CHMFindDialog* _fdlg;
 
 private:
@@ -180,3 +179,17 @@ private:
 
 #endif // __CHMHTMLWINDOW_H_
 
+
+/*
+  Local Variables:
+  mode: c++
+  c-basic-offset: 8
+  tab-width: 8
+  c-indent-comments-syntactically-p: t
+  c-tab-always-indent: t
+  indent-tabs-mode: t
+  End:
+*/
+
+// vim:shiftwidth=8:autoindent:tabstop=8:noexpandtab:softtabstop=8
+
